//
// OMNeT++ Arrival Process Module Example Package
//
// Author: Jan Kriege (Department of Computer Science, TU Dortmund, jan.kriege@tu-dortmund.de)
//
//
// This program is free software; you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation; either version 2 of the License, or
// (at your option) any later version.
// 
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
// 
// You should have received a copy of the GNU General Public License along
// with this program; if not, write to the Free Software Foundation, Inc.,
// 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
//

#include "Server.h"

Define_Module(Server);

Server::Server() {
	endOfServiceMessage = NULL;
	currentJob = NULL;
}

Server::~Server() {
	delete currentJob;
	cancelAndDelete(endOfServiceMessage);
}

void Server::initialize() {
	endOfServiceMessage = new cMessage("EoS");
    queue.setName("queue");
    lastTime = 0.0;
    buffer = par("buffer").longValue();
}

void Server::handleMessage(cMessage *msg) {
    if (msg==endOfServiceMessage) {
		// Job has finished service
    	EV << "Completed service of " << currentJob->getName() << endl;
    	send(currentJob, "out");
        if (!queue.empty()) {
			// start next waiting job
        	currentJob = (cMessage*)queue.pop();
			EV << "Starting service of " << currentJob->getName() << endl;
			simtime_t  serviceTime = par("serviceTime");
            scheduleAt(simTime()+serviceTime, endOfServiceMessage);
            // update the statistics about the queue length
            if (queueLength.find(queue.length()+1) != queueLength.end()) {
            	queueLength[queue.length()+1] = queueLength[queue.length()+1] + simTime() - lastTime;
            } else {
            	queueLength[queue.length()+1] = simTime() - lastTime;
            }
            lastTime = simTime();
        } else {
        	// no job waiting in queue
        	currentJob = NULL;
        }
    } else {
		// arrival of new job
    	if (currentJob == NULL) {
    		// no job in service => start service for newly arrived job
    		currentJob = msg;
			EV << "Starting service of " << currentJob->getName() << endl;
			simtime_t  serviceTime = par("serviceTime");
			scheduleAt(simTime()+serviceTime, endOfServiceMessage);
    	} else {
    		// server is busy => put job in queue if possible
			if (queue.length() < buffer) {
				queue.insert(msg);
				// update the statistics about the queue length
				if (queueLength.find(queue.length()-1) != queueLength.end()) {
					queueLength[queue.length()-1] = queueLength[queue.length()-1] + simTime() - lastTime;
				} else {
					queueLength[queue.length()-1] = simTime() - lastTime;
				}
				lastTime = simTime();
			} else {
				// queue is full => job is lost
				delete msg;
			}
    	}
    }
}

void Server::finish() {
	if (queueLength.find(queue.length()) != queueLength.end()) {
		queueLength[queue.length()] = queueLength[queue.length()] + simTime() - lastTime;
	} else {
		queueLength[queue.length()] = simTime() - lastTime;
	}
	simtime_t mean_ql = 0.0;
	// record the queue length distribution ...
	for (std::map<unsigned int, simtime_t>::iterator i=queueLength.begin(); i != queueLength.end(); i++) {
		std::stringstream str;
		str << i->first;
		recordScalar(str.str().c_str(), (i->second)/simTime());
		mean_ql = mean_ql + i->first * i->second / simTime();
	}
	// ... and the mean queue length
	recordScalar("Mean queue length", mean_ql);
}

