//
// OMNeT++ Arrival Process Module Example Package
//
// Author: Jan Kriege (Department of Computer Science, TU Dortmund, jan.kriege@tu-dortmund.de)
//
//
// This program is free software; you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation; either version 2 of the License, or
// (at your option) any later version.
// 
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
// 
// You should have received a copy of the GNU General Public License along
// with this program; if not, write to the Free Software Foundation, Inc.,
// 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
//

#include "TraceArrival.h"
#include <fstream>

using namespace std;

Define_Module(TraceArrival);

TraceArrival::TraceArrival() {
	sendMessageEvent = NULL;
	this->trace = NULL;
	number = 0;
}

TraceArrival::~TraceArrival() {
	cancelAndDelete(sendMessageEvent);
	delete this->trace;
}

void TraceArrival::initialize() {
	// load trace
	this->trace = new Trace(par("tracefile").stdstringValue());
	sendMessageEvent = new cMessage("sendMessageEvent");
	scheduleAt(simTime(), sendMessageEvent);
}

void TraceArrival::handleMessage(cMessage *msg) {
	ASSERT(msg==sendMessageEvent);
	// generate arrival
	cMessage *arrival = new cMessage("arrival");
	send(arrival, "out");
	number++;
	// determine next arrival time
	double nextInterarrivalTime = this->trace->getNextElement();
	// schedule next arrival
	scheduleAt(simTime()+nextInterarrivalTime, sendMessageEvent);
}

void TraceArrival::finish() {
    EV << "Total jobs generated: " << number << endl;
}

// -----------------------------------------------------------------------------------

Trace::Trace(std::string file) {
	this->pos = 0;
	this->trace.clear();
	this->load(file);
}

Trace::~Trace() {
}

void Trace::load(std::string file) {
	// delete old entries
	this->trace.clear();
	// open file
	std::ifstream tfile;
	tfile.open(file.c_str());
	if (!tfile.is_open()) {
		throw cException("Cannot open trace file.");
	}
	double val;
	std::string line;
	// read trace data
	while (getline(tfile, line)) {
		std::stringstream i;
		//skip empty lines and comments
		if ((line != "") && (line[0] != '#')) {
			i << line;
			i >> val;
			trace.push_back(val);
		}
	}
	tfile.close();
}

double Trace::getNextElement() {
	if (this->pos >= this->trace.size()) {
		EV << "WARNING: End of trace reached." << endl;
		this->pos = 0;
	}
	this->pos++;
	return (this->trace[this->pos-1]);
}
