//
// OMNeT++ Arrival Process Module Example Package
//
// Author: Jan Kriege (Department of Computer Science, TU Dortmund, jan.kriege@tu-dortmund.de)
//
//
// This program is free software; you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation; either version 2 of the License, or
// (at your option) any later version.
// 
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
// 
// You should have received a copy of the GNU General Public License along
// with this program; if not, write to the Free Software Foundation, Inc.,
// 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
//

#ifndef __TRACEARRIVAL_H__
#define __TRACEARRIVAL_H__

#include "omnetpp.h"
#include <vector>

using namespace omnetpp;

// Utility class for loading a trace file and accessing its elements.
class Trace {
	public:

		// Creates a Trace object.
		// Parameters:
		// 		file		path to trace file
		Trace(std::string file);

		// Dsetructor
		~Trace();

		// Returns the next element of the trace
		double getNextElement();

	private:

		// Load a trace file.
		// Parameters:
		// 		file		path to trace file
		void load(std::string file);

		// Vector with trace elements
		std::vector<double> trace;

		// Current element of trace
		unsigned int pos;
};

// Generates arrival events from a trace file
class TraceArrival : public cSimpleModule {
	public:
		TraceArrival();
		virtual ~TraceArrival();
	private:
		cMessage *sendMessageEvent;
		// Trace
		Trace* trace;
		// Number of generated events
		unsigned int number;
	protected:
		virtual void initialize();
		virtual void handleMessage(cMessage *msg);
		virtual void finish();
};

#endif
